import java.util.LinkedList;
import java.util.Vector;

public class BST {

	int value;
	BST left, right;

	public BST(int v) {
		this(null, v, null);
	}

	public BST(BST l, int v, BST r) {
		value = v;
		left = l;
		right = r;
	}

	static BST insert(BST a, int x) {
		if (a == null)
			return new BST(x);
		if (a.value > x)
			a.left = insert(a.left, x);
		if (a.value < x)
			a.right = insert(a.right, x);
		return a;
	}
	
	static int getMin(BST a) {
        assert (a != null) : "BST.getMin may not be called with a null argument";
		if (a.left == null)
			return a.value;
		return getMin(a.left);
	}

	static BST removeMin(BST a) {
        assert (a != null) : "BST.removeMin may not be called with a null argument";
		if (a.left == null)
			return a.right;
		return new BST(removeMin(a.left), a.value, a.right);
	}

	// q11
	static BST merge(BST s1, BST s2) {
		if (s1 == null)
			return s2;
		if (s2 == null)
			return s1;
		return new BST(s1, getMin(s2), removeMin(s2));
	}

	// q12
	static LinkedList<Integer> toList(BST s) {
		LinkedList<Integer> l = new LinkedList<Integer>();
		toList(l, s);
		return l;
	}

	// q12
	private static void toList(LinkedList<Integer> l, BST s) {
		if (s == null)
			return;
		toList(l, s.left);
		l.add(s.value);
		toList(l, s.right);
	}

	// q13
	static Vector<Pair<BST>> allSubsets(BST a) {
		Vector<Pair<BST>> all = new Vector<Pair<BST>>();
		if (a == null) {
			all.add(new Pair<BST>(null, null));
		} else {
			for (Pair<BST> l : allSubsets(a.left)) {
				for (Pair<BST> r : allSubsets(a.right)) {
					BST s1 = new BST(l.first, a.value, r.first);
					BST s1bar = merge(l.second, r.second);
					all.add(new Pair<BST>(s1, s1bar));
					all.add(new Pair<BST>(s1bar, s1));
				}
			}
		}
		return all;
	}

	// q3
	private static int computeHashCode(BST s, int hash) {
		if (s == null)
			return hash;
		hash = computeHashCode(s.left, hash);
		hash = 19 * hash + s.value;
		hash = computeHashCode(s.right, hash);
		return hash;
	}

	// q3
	static int computeHashCode(BST s) {
		return computeHashCode(s, 0);
	}

	// q3
	static boolean areEqual(BST a, BST b) {
		return Enum.areEqual(Enum.build(a, null), Enum.build(b, null));
	}
	
}
